@@ -40,23 +40,24 @@
 # - View source doesn't work on windows unless you install the less program (From GnuUtils or Cygwin)
 
 
-import platform
-import os
-import sys
 import curses
-import math
-import re
-import time
+import errno
 import functools
-
+import math
+import os
+import platform
+import re
 import struct
+import sys
+import time
+from typing import Iterator, NoReturn, List
+import unicodedata
+from dataclasses import dataclass
 
 if platform.system() != "Windows":
     import signal  # Windows does not have job control
     import termios  # Windows uses curses
     import fcntl  # Windows uses curses
-import unicodedata
-import errno
 
 
 # These are used for syntax highlighting
@@ -67,7 +68,7 @@ from pygments.token import Token
 from .formatter import BPythonFormatter
 
 # This for config
-from .config import Struct, getpreferredencoding
+from .config import getpreferredencoding
 
 # This for keys
 from .keys import cli_key_dispatch as key_dispatch
@@ -90,7 +91,14 @@ DO_RESIZE = False
 # ---
 
 
-def calculate_screen_lines(tokens, width, cursor=0):
+@dataclass
+class ShowListState:
+    cols: int = 0
+    rows: int = 0
+    wl: int = 0
+
+
+def calculate_screen_lines(tokens, width, cursor=0) -> int:
     """Given a stream of tokens and a screen width plus an optional
     initial cursor position, return the amount of needed lines on the
     screen."""
@@ -123,51 +131,51 @@ class FakeStream:
     provided."""
 
     def __init__(self, interface, get_dest):
-        self.encoding = getpreferredencoding()
+        self.encoding: str = getpreferredencoding()
         self.interface = interface
         self.get_dest = get_dest
 
     @forward_if_not_current
-    def write(self, s):
+    def write(self, s) -> None:
         self.interface.write(s)
 
     @forward_if_not_current
-    def writelines(self, l):
+    def writelines(self, l) -> None:
         for s in l:
             self.write(s)
 
-    def isatty(self):
+    def isatty(self) -> bool:
         # some third party (amongst them mercurial) depend on this
         return True
 
-    def flush(self):
+    def flush(self) -> None:
         self.interface.flush()
 
 
 class FakeStdin:
     """Provide a fake stdin type for things like raw_input() etc."""
 
-    def __init__(self, interface):
+    def __init__(self, interface) -> None:
         """Take the curses Repl on init and assume it provides a get_key method
         which, fortunately, it does."""
 
         self.encoding = getpreferredencoding()
         self.interface = interface
-        self.buffer = list()
+        self.buffer: List[str] = list()
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator:
         return iter(self.readlines())
 
     def flush(self):
         """Flush the internal buffer. This is a no-op. Flushing stdin
         doesn't make any sense anyway."""
 
-    def write(self, value):
+    def write(self, value) -> NoReturn:
         # XXX IPython expects sys.stdin.write to exist, there will no doubt be
         # others, so here's a hack to keep them happy
         raise OSError(errno.EBADF, "sys.stdin is read-only")
 
-    def isatty(self):
+    def isatty(self) -> bool:
         return True
 
     def readline(self, size=-1):
@@ -1254,11 +1262,7 @@ class CLIRepl(repl.Repl):
     def show_list(
         self, items, arg_pos, topline=None, formatter=None, current_item=None
     ):
-
-        shared = Struct()
-        shared.cols = 0
-        shared.rows = 0
-        shared.wl = 0
+        shared = ShowListState()
         y, x = self.scr.getyx()
         h, w = self.scr.getmaxyx()
         down = y < h // 2
