@@ -1,7 +1,7 @@
-import blessings
+import code
 import contextlib
 import errno
-import greenlet
+import itertools
 import logging
 import os
 import re
@@ -11,13 +11,13 @@ import sys
 import tempfile
 import time
 import unicodedata
+from enum import Enum
+from types import TracebackType
+from typing import Dict, Any, List, Optional, Tuple, Union, cast, Type
+from typing_extensions import Literal
 
-from pygments import format as pygformat
-from pygments.lexers import Python3Lexer
-from pygments.formatters import TerminalFormatter
-
-from cwcwidth import wcswidth
-
+import blessings
+import greenlet
 from curtsies import (
     FSArray,
     fmtstr,
@@ -29,34 +29,35 @@ from curtsies import (
 )
 from curtsies.configfile_keynames import keymap as key_dispatch
 from curtsies.input import is_main_thread
-
-from bpython import __version__
-from bpython.repl import (
-    Repl as BpythonRepl,
-    SourceNotFound,
-    LineTypeTranslator as LineType,
-)
-from bpython.config import getpreferredencoding
-from bpython.formatter import BPythonFormatter
-from bpython import autocomplete
-from bpython.translations import _
-from bpython.pager import get_pager_command
+from cwcwidth import wcswidth
+from pygments import format as pygformat
+from pygments.formatters import TerminalFormatter
+from pygments.lexers import Python3Lexer
 
 from . import events as bpythonevents, sitefix, replpainter as paint
+from ..config import Config
 from .coderunner import (
     CodeRunner,
     FakeOutput,
 )
 from .filewatch import ModuleChangedEventHandler
 from .interaction import StatusBar
-from .manual_readline import edit_keys
-from .parse import parse as bpythonparse, func_for_letter, color_for_letter
-from .preprocess import preprocess
 from .interpreter import (
     Interp,
     code_finished_will_parse,
 )
-
+from .manual_readline import edit_keys
+from .parse import parse as bpythonparse, func_for_letter, color_for_letter
+from .preprocess import preprocess
+from .. import __version__
+from ..config import getpreferredencoding
+from ..formatter import BPythonFormatter
+from ..pager import get_pager_command
+from ..repl import (
+    Repl,
+    SourceNotFound,
+)
+from ..translations import _
 
 logger = logging.getLogger(__name__)
 
@@ -74,6 +75,14 @@ EDIT_SESSION_HEADER = """### current bpython session - make changes and save to
 MAX_EVENTS_POSSIBLY_NOT_PASTE = 20
 
 
+class LineType(Enum):
+    """Used when adding a tuple to all_logical_lines, to get input / output values
+    having to actually type/know the strings"""
+
+    INPUT = "input"
+    OUTPUT = "output"
+
+
 class FakeStdin:
     """The stdin object user code will reference
 
@@ -93,7 +102,7 @@ class FakeStdin:
         else:
             self.rl_char_sequences = edit_keys
 
-    def process_event(self, e):
+    def process_event(self, e: Union[events.Event, str]) -> None:
         assert self.has_focus
 
         logger.debug("fake input processing event %r", e)
@@ -111,8 +120,6 @@ class FakeStdin:
             self.current_line = ""
             self.cursor_offset = 0
             self.repl.run_code_and_maybe_finish()
-        elif e in ("<Esc+.>",):
-            self.get_last_word()
         elif e in ("<ESC>",):
             pass
         elif e in ("<Ctrl-d>",):
@@ -271,7 +278,15 @@ class ImportFinder:
             return ImportLoader(self.watcher, loader)
 
 
-class BaseRepl(BpythonRepl):
+def _process_ps(ps, default_ps: str):
+    """Replace ps1/ps2 with the default if the user specified value contains control characters."""
+    if not isinstance(ps, str):
+        return ps
+
+    return ps if wcswidth(ps) >= 0 else default_ps
+
+
+class BaseRepl(Repl):
     """Python Repl
 
     Reacts to events like
@@ -295,11 +310,11 @@ class BaseRepl(BpythonRepl):
 
     def __init__(
         self,
-        config,
-        locals_=None,
-        banner=None,
-        interp=None,
-        orig_tcattrs=None,
+        config: Config,
+        locals_: Dict[str, Any] = None,
+        banner: str = None,
+        interp: code.InteractiveInterpreter = None,
+        orig_tcattrs: List[Any] = None,
     ):
         """
         locals_ is a mapping of locals to pass into the interpreter
@@ -317,7 +332,7 @@ class BaseRepl(BpythonRepl):
 
         if interp is None:
             interp = Interp(locals=locals_)
-            interp.write = self.send_to_stdouterr
+            interp.write = self.send_to_stdouterr  # type: ignore
         if banner is None:
             if config.help_key:
                 banner = (
@@ -327,8 +342,6 @@ class BaseRepl(BpythonRepl):
                 )
             else:
                 banner = None
-        # only one implemented currently
-        config.autocomplete_mode = autocomplete.AutocompleteModes.SIMPLE
         if config.cli_suggestion_width <= 0 or config.cli_suggestion_width > 1:
             config.cli_suggestion_width = 1
 
@@ -356,12 +369,12 @@ class BaseRepl(BpythonRepl):
         self._current_line = ""
 
         # current line of output - stdout and stdin go here
-        self.current_stdouterr_line = ""
+        self.current_stdouterr_line = ""  # Union[str, FmtStr]
 
         # this is every line that's been displayed (input and output)
         # as with formatting applied. Logical lines that exceeded the terminal width
         # at the time of output are split across multiple entries in this list.
-        self.display_lines = []
+        self.display_lines: List[FmtStr] = []
 
         # this is every line that's been executed; it gets smaller on rewind
         self.history = []
@@ -372,11 +385,11 @@ class BaseRepl(BpythonRepl):
         #   - the first element the line (string, not fmtsr)
         #   - the second element is one of 2 global constants: "input" or "output"
         #     (use LineType.INPUT or LineType.OUTPUT to avoid typing these strings)
-        self.all_logical_lines = []
+        self.all_logical_lines: List[Tuple[str, LineType]] = []
 
         # formatted version of lines in the buffer kept around so we can
         # unhighlight parens using self.reprint_line as called by bpython.Repl
-        self.display_buffer = []
+        self.display_buffer: List[FmtStr] = []
 
         # how many times display has been scrolled down
         # because there wasn't room to display everything
@@ -385,7 +398,7 @@ class BaseRepl(BpythonRepl):
         # cursor position relative to start of current_line, 0 is first char
         self._cursor_offset = 0
 
-        self.orig_tcattrs = orig_tcattrs
+        self.orig_tcattrs: Optional[List[Any]] = orig_tcattrs
 
         self.coderunner = CodeRunner(self.interp, self.request_refresh)
 
@@ -417,7 +430,7 @@ class BaseRepl(BpythonRepl):
         # some commands act differently based on the prev event
         # this list doesn't include instances of event.Event,
         # only keypress-type events (no refresh screen events etc.)
-        self.last_events = [None] * 50
+        self.last_events: List[Optional[str]] = [None] * 50
 
         # displays prev events in a column on the right hand side
         self.presentation_mode = False
@@ -435,8 +448,10 @@ class BaseRepl(BpythonRepl):
 
         self.original_modules = set(sys.modules.keys())
 
-        self.width = None
-        self.height = None
+        # as long as the first event received is a window resize event,
+        # this works fine...
+        self.width: int = cast(int, None)
+        self.height: int = cast(int, None)
 
         self.status_bar.message(banner)
 
@@ -459,7 +474,7 @@ class BaseRepl(BpythonRepl):
         """Returns the current width and height of the display area."""
         return (50, 10)
 
-    def _schedule_refresh(self, when="now"):
+    def _schedule_refresh(self, when: float):
         """Arrange for the bpython display to be refreshed soon.
 
         This method will be called when the Repl wants the display to be
@@ -559,7 +574,12 @@ class BaseRepl(BpythonRepl):
         sitefix.monkeypatch_quit()
         return self
 
-    def __exit__(self, *args):
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> Literal[False]:
         sys.stdin = self.orig_stdin
         sys.stdout = self.orig_stdout
         sys.stderr = self.orig_stderr
@@ -570,6 +590,7 @@ class BaseRepl(BpythonRepl):
             signal.signal(signal.SIGTSTP, self.orig_sigtstp_handler)
 
         sys.meta_path = self.orig_meta_path
+        return False
 
     def sigwinch_handler(self, signum, frame):
         old_rows, old_columns = self.height, self.width
@@ -602,7 +623,7 @@ class BaseRepl(BpythonRepl):
         self.unhighlight_paren()
 
     # Event handling
-    def process_event(self, e):
+    def process_event(self, e: Union[events.Event, str]) -> Optional[bool]:
         """Returns True if shutting down, otherwise returns None.
         Mostly mutates state of Repl object"""
 
@@ -612,9 +633,10 @@ class BaseRepl(BpythonRepl):
         else:
             self.last_events.append(e)
             self.last_events.pop(0)
-            return self.process_key_event(e)
+            self.process_key_event(e)
+            return None
 
-    def process_control_event(self, e):
+    def process_control_event(self, e) -> Optional[bool]:
 
         if isinstance(e, bpythonevents.ScheduledRefreshRequestEvent):
             # This is a scheduled refresh - it's really just a refresh (so nop)
@@ -629,7 +651,7 @@ class BaseRepl(BpythonRepl):
                 self.run_code_and_maybe_finish()
 
         elif self.status_bar.has_focus:
-            return self.status_bar.process_event(e)
+            self.status_bar.process_event(e)
 
         # handles paste events for both stdin and repl
         elif isinstance(e, events.PasteEvent):
@@ -669,12 +691,11 @@ class BaseRepl(BpythonRepl):
             self.undo(n=e.n)
 
         elif self.stdin.has_focus:
-            return self.stdin.process_event(e)
+            self.stdin.process_event(e)
 
         elif isinstance(e, events.SigIntEvent):
             logger.debug("received sigint event")
             self.keyboard_interrupt()
-            return
 
         elif isinstance(e, bpythonevents.ReloadEvent):
             if self.watching_files:
@@ -686,8 +707,9 @@ class BaseRepl(BpythonRepl):
 
         else:
             raise ValueError("Don't know how to handle event type: %r" % e)
+        return None
 
-    def process_key_event(self, e):
+    def process_key_event(self, e: str) -> None:
         # To find the curtsies name for a keypress, try
         # python -m curtsies.events
         if self.status_bar.has_focus:
@@ -742,7 +764,7 @@ class BaseRepl(BpythonRepl):
         elif e in key_dispatch[self.config.reimport_key]:
             self.clear_modules_and_reevaluate()
         elif e in key_dispatch[self.config.toggle_file_watch_key]:
-            return self.toggle_file_watch()
+            self.toggle_file_watch()
         elif e in key_dispatch[self.config.clear_screen_key]:
             self.request_paint_to_clear_screen = True
         elif e in key_dispatch[self.config.show_source_key]:
@@ -993,9 +1015,9 @@ class BaseRepl(BpythonRepl):
             )
             return
         lines = text.split("\n")
-        if not lines[-1].strip():
+        if len(lines) and not lines[-1].strip():
             lines.pop()  # strip last line if empty
-        if lines[-1].startswith("### "):
+        if len(lines) and lines[-1].startswith("### "):
             current_line = lines[-1][4:]
         else:
             current_line = ""
@@ -1242,8 +1264,6 @@ class BaseRepl(BpythonRepl):
 
         Must be able to handle FmtStrs because interpreter pass in
         tracebacks already formatted."""
-        if not output:
-            return
         lines = output.split("\n")
         logger.debug("display_lines: %r", self.display_lines)
         self.current_stdouterr_line += lines[0]
@@ -1263,7 +1283,9 @@ class BaseRepl(BpythonRepl):
                 )
             )
             # These can be FmtStrs, but self.all_logical_lines only wants strings
-            for line in [self.current_stdouterr_line] + lines[1:-1]:
+            for line in itertools.chain(
+                (self.current_stdouterr_line,), lines[1:-1]
+            ):
                 if isinstance(line, FmtStr):
                     self.all_logical_lines.append((line.s, LineType.OUTPUT))
                 else:
@@ -1418,7 +1440,7 @@ class BaseRepl(BpythonRepl):
         user_quit=False,
         try_preserve_history_height=30,
         min_infobox_height=5,
-    ):
+    ) -> Tuple[FSArray, Tuple[int, int]]:
         """Returns an array of min_height or more rows and width columns, plus
         cursor position
 
@@ -1571,8 +1593,19 @@ class BaseRepl(BpythonRepl):
         current_line_height = current_line_end_row - current_line_start_row
 
         if self.stdin.has_focus:
+            logger.debug(
+                "stdouterr when self.stdin has focus: %r %r",
+                type(self.current_stdouterr_line),
+                self.current_stdouterr_line,
+            )
+            # mypy can't do ternary type guards yet
+            stdouterr = self.current_stdouterr_line
+            if isinstance(stdouterr, FmtStr):
+                stdouterr_width = stdouterr.width
+            else:
+                stdouterr_width = len(stdouterr)
             cursor_row, cursor_column = divmod(
-                wcswidth(self.current_stdouterr_line)
+                stdouterr_width
                 + wcswidth(
                     self.stdin.current_line, max(0, self.stdin.cursor_offset)
                 ),
@@ -1581,13 +1614,13 @@ class BaseRepl(BpythonRepl):
             assert cursor_row >= 0 and cursor_column >= 0, (
                 cursor_row,
                 cursor_column,
+                self.current_stdouterr_line,
+                self.stdin.current_line,
             )
         elif self.coderunner.running:  # TODO does this ever happen?
             cursor_row, cursor_column = divmod(
-                (
-                    len(self.current_cursor_line_without_suggestion)
-                    + self.cursor_offset
-                ),
+                len(self.current_cursor_line_without_suggestion)
+                + self.cursor_offset,
                 width,
             )
             assert cursor_row >= 0 and cursor_column >= 0, (
@@ -1599,19 +1632,17 @@ class BaseRepl(BpythonRepl):
             )
         else:  # Common case for determining cursor position
             cursor_row, cursor_column = divmod(
-                (
-                    wcswidth(self.current_cursor_line_without_suggestion.s)
-                    - wcswidth(self.current_line)
-                    + wcswidth(self.current_line, max(0, self.cursor_offset))
-                )
+                wcswidth(self.current_cursor_line_without_suggestion.s)
+                - wcswidth(self.current_line)
+                + wcswidth(self.current_line, max(0, self.cursor_offset))
                 + self.number_of_padding_chars_on_current_cursor_line(),
                 width,
             )
             assert cursor_row >= 0 and cursor_column >= 0, (
                 cursor_row,
                 cursor_column,
-                len(self.current_cursor_line),
-                len(self.current_line),
+                self.current_cursor_line_without_suggestion.s,
+                self.current_line,
                 self.cursor_offset,
             )
         cursor_row += current_line_start_row
@@ -1720,17 +1751,21 @@ class BaseRepl(BpythonRepl):
             self.update_completion()
 
     def __repr__(self):
-        s = ""
-        s += "<" + repr(type(self)) + "\n"
-        s += " cursor_offset:" + repr(self.cursor_offset) + "\n"
-        s += " num display lines:" + repr(len(self.display_lines)) + "\n"
-        s += " lines scrolled down:" + repr(self.scroll_offset) + "\n"
-        s += ">"
-        return s
+        return f"""<{type(self)}
+  cursor_offset: {self.cursor_offset}
+  num display lines: {len(self.display_lines)}
+  lines scrolled down: {self.scroll_offset}
+>"""
 
-    def _get_current_line(self):
+    @property
+    def current_line(self):
+        """The current line"""
         return self._current_line
 
+    @current_line.setter
+    def current_line(self, value):
+        self._set_current_line(value)
+
     def _set_current_line(
         self,
         line,
@@ -1751,13 +1786,15 @@ class BaseRepl(BpythonRepl):
             self.special_mode = None
         self.unhighlight_paren()
 
-    current_line = property(
-        _get_current_line, _set_current_line, None, "The current line"
-    )
-
-    def _get_cursor_offset(self):
+    @property
+    def cursor_offset(self):
+        """The current cursor offset from the front of the "line"."""
         return self._cursor_offset
 
+    @cursor_offset.setter
+    def cursor_offset(self, value):
+        self._set_cursor_offset(value)
+
     def _set_cursor_offset(
         self,
         offset,
@@ -1780,13 +1817,6 @@ class BaseRepl(BpythonRepl):
             self.update_completion()
         self.unhighlight_paren()
 
-    cursor_offset = property(
-        _get_cursor_offset,
-        _set_cursor_offset,
-        None,
-        "The current cursor offset from the front of the " "line",
-    )
-
     def echo(self, msg, redraw=True):
         """
         Notification that redrawing the current line is necessary (we don't
@@ -1799,7 +1829,7 @@ class BaseRepl(BpythonRepl):
 
     @property
     def cpos(self):
-        "many WATs were had - it's the pos from the end of the line back" ""
+        "many WATs were had - it's the pos from the end of the line back"
         return len(self.current_line) - self.cursor_offset
 
     def reprint_line(self, lineno, tokens):
@@ -1837,7 +1867,7 @@ class BaseRepl(BpythonRepl):
             return self.take_back_empty_line()
 
         def prompt_for_undo():
-            n = BpythonRepl.prompt_undo(self)
+            n = super(BaseRepl, self).prompt_undo()
             if n > 0:
                 self.request_undo(n=n)
 
@@ -1914,7 +1944,7 @@ class BaseRepl(BpythonRepl):
         self._cursor_offset = 0
         self.current_line = ""
 
-    def initialize_interp(self):
+    def initialize_interp(self) -> None:
         self.coderunner.interp.locals["_repl"] = self
         self.coderunner.interp.runsource(
             "from bpython.curtsiesfrontend._internal import _Helper\n"
@@ -2019,11 +2049,10 @@ Press {config.edit_config_key} to edit this config file.
             "yank from buffer",
             "cut to buffer",
         )
-        pairs = []
-        pairs.append(
-            ["complete history suggestion", "right arrow at end of line"]
-        )
-        pairs.append(["previous match with current line", "up arrow"])
+        pairs = [
+            ["complete history suggestion", "right arrow at end of line"],
+            ["previous match with current line", "up arrow"],
+        ]
         for functionality, key in (
             (attr[:-4].replace("_", " "), getattr(self.config, attr))
             for attr in self.config.__dict__
@@ -2041,6 +2070,25 @@ Press {config.edit_config_key} to edit this config file.
             "{} : {}".format(func.rjust(max_func), key) for func, key in pairs
         )
 
+    def get_session_formatted_for_file(self) -> str:
+        def process():
+            for line, lineType in self.all_logical_lines:
+                if lineType == LineType.INPUT:
+                    yield line
+                elif line.rstrip():
+                    yield "# OUT: %s" % line
+            yield "### %s" % self.current_line
+
+        return "\n".join(process())
+
+    @property
+    def ps1(self):
+        return _process_ps(super().ps1, ">>> ")
+
+    @property
+    def ps2(self):
+        return _process_ps(super().ps2, "... ")
+
 
 def is_nop(char):
     return unicodedata.category(str(char)) == "Cc"
@@ -2051,7 +2099,8 @@ def tabs_to_spaces(line):
 
 
 def _last_word(line):
-    return line.split().pop() if line.split() else ""
+    split_line = line.split()
+    return split_line.pop() if split_line else ""
 
 
 def compress_paste_event(paste_event):
