@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # The MIT License
 #
 # Copyright (c) 2008 Bob Farrell
@@ -26,12 +24,21 @@
 # Pygments really kicks ass, it made it really easy to
 # get the exact behaviour I wanted, thanks Pygments.:)
 
-from __future__ import absolute_import
 
 from pygments.formatter import Formatter
-from pygments.token import Keyword, Name, Comment, String, Error, \
-    Number, Operator, Token, Whitespace, Literal, Punctuation
-from six import iteritems
+from pygments.token import (
+    Keyword,
+    Name,
+    Comment,
+    String,
+    Error,
+    Number,
+    Operator,
+    Token,
+    Whitespace,
+    Literal,
+    Punctuation,
+)
 
 """These format strings are pretty ugly.
 \x01 represents a colour marker, which
@@ -60,20 +67,21 @@ from six import iteritems
 Parenthesis = Token.Punctuation.Parenthesis
 
 theme_map = {
-    Keyword: 'keyword',
-    Name: 'name',
-    Comment: 'comment',
-    String: 'string',
-    Literal: 'string',
-    Error: 'error',
-    Number: 'number',
-    Token.Literal.Number.Float: 'number',
-    Operator: 'operator',
-    Punctuation: 'punctuation',
-    Token: 'token',
-    Whitespace: 'background',
-    Parenthesis: 'paren',
-    Parenthesis.UnderCursor: 'operator'}
+    Keyword: "keyword",
+    Name: "name",
+    Comment: "comment",
+    String: "string",
+    Literal: "string",
+    Error: "error",
+    Number: "number",
+    Token.Literal.Number.Float: "number",
+    Operator: "operator",
+    Punctuation: "punctuation",
+    Token: "token",
+    Whitespace: "background",
+    Parenthesis: "paren",
+    Parenthesis.UnderCursor: "operator",
+}
 
 
 class BPythonFormatter(Formatter):
@@ -90,23 +98,24 @@ class BPythonFormatter(Formatter):
 
     def __init__(self, color_scheme, **options):
         self.f_strings = {}
-        for k, v in iteritems(theme_map):
-            self.f_strings[k] = '\x01%s' % (color_scheme[v],)
+        for k, v in theme_map.items():
+            self.f_strings[k] = "\x01{}".format(color_scheme[v])
             if k is Parenthesis:
                 # FIXME: Find a way to make this the inverse of the current
                 # background colour
-                self.f_strings[k] += 'I'
-        Formatter.__init__(self, **options)
+                self.f_strings[k] += "I"
+        super().__init__(**options)
 
     def format(self, tokensource, outfile):
-        o = ''
+        o = ""
         for token, text in tokensource:
-            if text == '\n':
+            if text == "\n":
                 continue
 
             while token not in self.f_strings:
                 token = token.parent
-            o += "%s\x03%s\x04" % (self.f_strings[token], text)
+            o += "{}\x03{}\x04".format(self.f_strings[token], text)
         outfile.write(o.rstrip())
 
+
 # vim: sw=4 ts=4 sts=4 ai et
