@@ -1,7 +1,7 @@
 # The MIT License
 #
 # Copyright (c) 2009 the bpython authors.
-# Copyright (c) 2012,2015 Sebastian Ramacher
+# Copyright (c) 2012-2021 Sebastian Ramacher
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,8 @@
 
 import os
 import stat
-from itertools import islice
+from itertools import islice, chain
+from typing import Iterable, Optional, List, TextIO
 
 from .translations import _
 from .filelock import FileLock
@@ -32,7 +33,12 @@ from .filelock import FileLock
 class History:
     """Stores readline-style history and current place in it"""
 
-    def __init__(self, entries=None, duplicates=True, hist_size=100):
+    def __init__(
+        self,
+        entries: Optional[Iterable[str]] = None,
+        duplicates: bool = True,
+        hist_size: int = 100,
+    ) -> None:
         if entries is None:
             self.entries = [""]
         else:
@@ -45,10 +51,10 @@ class History:
         self.duplicates = duplicates
         self.hist_size = hist_size
 
-    def append(self, line):
+    def append(self, line: str) -> None:
         self.append_to(self.entries, line)
 
-    def append_to(self, entries, line):
+    def append_to(self, entries: List[str], line: str) -> None:
         line = line.rstrip("\n")
         if line:
             if not self.duplicates:
@@ -60,15 +66,19 @@ class History:
                     pass
             entries.append(line)
 
-    def first(self):
+    def first(self) -> str:
         """Move back to the beginning of the history."""
         if not self.is_at_end:
             self.index = len(self.entries)
         return self.entries[-self.index]
 
     def back(
-        self, start=True, search=False, target=None, include_current=False
-    ):
+        self,
+        start: bool = True,
+        search: bool = False,
+        target: Optional[str] = None,
+        include_current: bool = False,
+    ) -> str:
         """Move one step back in the history."""
         if target is None:
             target = self.saved_line
@@ -84,15 +94,17 @@ class History:
         return self.entry
 
     @property
-    def entry(self):
+    def entry(self) -> str:
         """The current entry, which may be the saved line"""
         return self.entries[-self.index] if self.index else self.saved_line
 
     @property
-    def entries_by_index(self):
-        return list(reversed(self.entries + [self.saved_line]))
+    def entries_by_index(self) -> List[str]:
+        return list(chain((self.saved_line,), reversed(self.entries)))
 
-    def find_match_backward(self, search_term, include_current=False):
+    def find_match_backward(
+        self, search_term: str, include_current: bool = False
+    ) -> int:
         add = 0 if include_current else 1
         start = self.index + add
         for idx, val in enumerate(islice(self.entries_by_index, start, None)):
@@ -100,7 +112,9 @@ class History:
                 return idx + add
         return 0
 
-    def find_partial_match_backward(self, search_term, include_current=False):
+    def find_partial_match_backward(
+        self, search_term: str, include_current: bool = False
+    ) -> int:
         add = 0 if include_current else 1
         start = self.index + add
         for idx, val in enumerate(islice(self.entries_by_index, start, None)):
@@ -109,8 +123,12 @@ class History:
         return 0
 
     def forward(
-        self, start=True, search=False, target=None, include_current=False
-    ):
+        self,
+        start: bool = True,
+        search: bool = False,
+        target: Optional[str] = None,
+        include_current: bool = False,
+    ) -> str:
         """Move one step forward in the history."""
         if target is None:
             target = self.saved_line
@@ -128,7 +146,9 @@ class History:
             self.index = 0
             return self.saved_line
 
-    def find_match_forward(self, search_term, include_current=False):
+    def find_match_forward(
+        self, search_term: str, include_current: bool = False
+    ) -> int:
         add = 0 if include_current else 1
         end = max(0, self.index - (1 - add))
         for idx in range(end):
@@ -137,7 +157,9 @@ class History:
                 return idx + (0 if include_current else 1)
         return self.index
 
-    def find_partial_match_forward(self, search_term, include_current=False):
+    def find_partial_match_forward(
+        self, search_term: str, include_current: bool = False
+    ) -> int:
         add = 0 if include_current else 1
         end = max(0, self.index - (1 - add))
         for idx in range(end):
@@ -146,40 +168,40 @@ class History:
                 return idx + add
         return self.index
 
-    def last(self):
+    def last(self) -> str:
         """Move forward to the end of the history."""
         if not self.is_at_start:
             self.index = 0
         return self.entries[0]
 
     @property
-    def is_at_end(self):
+    def is_at_end(self) -> bool:
         return self.index >= len(self.entries) or self.index == -1
 
     @property
-    def is_at_start(self):
+    def is_at_start(self) -> bool:
         return self.index == 0
 
-    def enter(self, line):
+    def enter(self, line: str) -> None:
         if self.index == 0:
             self.saved_line = line
 
-    def reset(self):
+    def reset(self) -> None:
         self.index = 0
         self.saved_line = ""
 
-    def load(self, filename, encoding):
+    def load(self, filename: str, encoding: str) -> None:
         with open(filename, encoding=encoding, errors="ignore") as hfile:
             with FileLock(hfile, filename=filename):
                 self.entries = self.load_from(hfile)
 
-    def load_from(self, fd):
-        entries = []
+    def load_from(self, fd: TextIO) -> List[str]:
+        entries: List[str] = []
         for line in fd:
             self.append_to(entries, line)
         return entries if len(entries) else [""]
 
-    def save(self, filename, encoding, lines=0):
+    def save(self, filename: str, encoding: str, lines: int = 0) -> None:
         fd = os.open(
             filename,
             os.O_WRONLY | os.O_CREAT | os.O_TRUNC,
@@ -189,14 +211,18 @@ class History:
             with FileLock(hfile, filename=filename):
                 self.save_to(hfile, self.entries, lines)
 
-    def save_to(self, fd, entries=None, lines=0):
+    def save_to(
+        self, fd: TextIO, entries: Optional[List[str]] = None, lines: int = 0
+    ) -> None:
         if entries is None:
             entries = self.entries
         for line in entries[-lines:]:
             fd.write(line)
             fd.write("\n")
 
-    def append_reload_and_write(self, s, filename, encoding):
+    def append_reload_and_write(
+        self, s: str, filename: str, encoding: str
+    ) -> None:
         if not self.hist_size:
             return self.append(s)
 
