@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # The MIT License
 #
 # Copyright (c) 2009-2011 the bpython authors.
@@ -23,26 +21,20 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-from __future__ import absolute_import
-
 import inspect
-import io
 import keyword
 import pydoc
+import re
 from collections import namedtuple
-from six.moves import range
+from typing import Any, Optional, Type
+from types import MemberDescriptorType, TracebackType
+from typing_extensions import Literal
 
 from pygments.token import Token
-from types import MemberDescriptorType
+from pygments.lexers import Python3Lexer
 
-from ._py3compat import PythonLexer, py3
 from .lazyre import LazyReCompile
 
-if not py3:
-    import types
-
-    _name = LazyReCompile(r"[a-zA-Z_]\w*$")
-
 ArgSpec = namedtuple(
     "ArgSpec",
     [
@@ -59,11 +51,11 @@ ArgSpec = namedtuple(
 FuncProps = namedtuple("FuncProps", ["func", "argspec", "is_bound_method"])
 
 
-class AttrCleaner(object):
+class AttrCleaner:
     """A context manager that tries to make an object not exhibit side-effects
-       on attribute lookup."""
+    on attribute lookup."""
 
-    def __init__(self, obj):
+    def __init__(self, obj: Any) -> None:
         self.obj = obj
 
     def __enter__(self):
@@ -81,24 +73,28 @@ class AttrCleaner(object):
         # original methods. :-(
         # The upshot being that introspecting on an object to display its
         # attributes will avoid unwanted side-effects.
-        if is_new_style(self.obj):
-            __getattr__ = getattr(type_, "__getattr__", None)
-            if __getattr__ is not None:
-                try:
-                    setattr(type_, "__getattr__", (lambda *_, **__: None))
-                except TypeError:
-                    __getattr__ = None
-            __getattribute__ = getattr(type_, "__getattribute__", None)
-            if __getattribute__ is not None:
-                try:
-                    setattr(type_, "__getattribute__", object.__getattribute__)
-                except TypeError:
-                    # XXX: This happens for e.g. built-in types
-                    __getattribute__ = None
+        __getattr__ = getattr(type_, "__getattr__", None)
+        if __getattr__ is not None:
+            try:
+                setattr(type_, "__getattr__", (lambda *_, **__: None))
+            except TypeError:
+                __getattr__ = None
+        __getattribute__ = getattr(type_, "__getattribute__", None)
+        if __getattribute__ is not None:
+            try:
+                setattr(type_, "__getattribute__", object.__getattribute__)
+            except TypeError:
+                # XXX: This happens for e.g. built-in types
+                __getattribute__ = None
         self.attribs = (__getattribute__, __getattr__)
         # /Dark magic
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> Literal[False]:
         """Restore an object's magic methods."""
         type_ = type(self.obj)
         __getattribute__, __getattr__ = self.attribs
@@ -108,22 +104,10 @@ class AttrCleaner(object):
         if __getattr__ is not None:
             setattr(type_, "__getattr__", __getattr__)
         # /Dark magic
+        return False
 
 
-if py3:
-
-    def is_new_style(obj):
-        return True
-
-
-else:
-
-    def is_new_style(obj):
-        """Returns True if obj is a new-style class or object"""
-        return type(obj) not in [types.InstanceType, types.ClassType]
-
-
-class _Repr(object):
+class _Repr:
     """
     Helper for `fixlongargs()`: Returns the given value in `__repr__()`.
     """
@@ -138,21 +122,21 @@ class _Repr(object):
 
 
 def parsekeywordpairs(signature):
-    tokens = PythonLexer().get_tokens(signature)
+    tokens = Python3Lexer().get_tokens(signature)
     preamble = True
     stack = []
     substack = []
     parendepth = 0
     for token, value in tokens:
         if preamble:
-            if token is Token.Punctuation and value == u"(":
+            if token is Token.Punctuation and value == "(":
                 preamble = False
             continue
 
         if token is Token.Punctuation:
-            if value in [u"(", u"{", u"["]:
+            if value in ("(", "{", "["):
                 parendepth += 1
-            elif value in [u")", u"}", u"]"]:
+            elif value in (")", "}", "]"):
                 parendepth -= 1
             elif value == ":" and parendepth == -1:
                 # End of signature reached
@@ -167,11 +151,7 @@ def parsekeywordpairs(signature):
         if value and (parendepth > 0 or value.strip()):
             substack.append(value)
 
-    d = {}
-    for item in stack:
-        if len(item) >= 3:
-            d[item[0]] = "".join(item[2:])
-    return d
+    return {item[0]: "".join(item[2:]) for item in stack if len(item) >= 3}
 
 
 def fixlongargs(f, argspec):
@@ -188,7 +168,7 @@ def fixlongargs(f, argspec):
     keys = argspec[0][-len(values) :]
     try:
         src = inspect.getsourcelines(f)
-    except (IOError, IndexError):
+    except (OSError, IndexError):
         # IndexError is raised in inspect.findsource(), can happen in
         # some situations. See issue #94.
         return
@@ -202,7 +182,9 @@ def fixlongargs(f, argspec):
     argspec[3] = values
 
 
-getpydocspec_re = LazyReCompile(r"([a-zA-Z_][a-zA-Z0-9_]*?)\((.*?)\)")
+getpydocspec_re = LazyReCompile(
+    r"([a-zA-Z_][a-zA-Z0-9_]*?)\((.*?)\)", re.DOTALL
+)
 
 
 def getpydocspec(f, func):
@@ -218,10 +200,10 @@ def getpydocspec(f, func):
     if not hasattr_safe(f, "__name__") or s.groups()[0] != f.__name__:
         return None
 
-    args = list()
-    defaults = list()
+    args = []
+    defaults = []
     varargs = varkwargs = None
-    kwonly_args = list()
+    kwonly_args = []
     kwonly_defaults = dict()
     for arg in s.group(2).split(","):
         arg = arg.strip()
@@ -229,6 +211,9 @@ def getpydocspec(f, func):
             varkwargs = arg[2:]
         elif arg.startswith("*"):
             varargs = arg[1:]
+        elif arg == "...":
+            # At least print denotes "..." as separator between varargs and kwonly args.
+            varargs = ""
         else:
             arg, _, default = arg.partition("=")
             if varargs is not None:
@@ -266,11 +251,7 @@ def getfuncprops(func, f):
         # '__init__' throws xmlrpclib.Fault (see #202)
         return None
     try:
-        if py3:
-            argspec = get_argspec_from_signature(f)
-        else:
-            argspec = list(inspect.getargspec(f))
-
+        argspec = get_argspec_from_signature(f)
         fixlongargs(f, argspec)
         if len(argspec) == 4:
             argspec = argspec + [list(), dict(), None]
@@ -287,20 +268,10 @@ def getfuncprops(func, f):
 
 
 def is_eval_safe_name(string):
-    if py3:
-        return all(
-            part.isidentifier() and not keyword.iskeyword(part)
-            for part in string.split(".")
-        )
-    else:
-        return all(
-            _name.match(part) and not keyword.iskeyword(part)
-            for part in string.split(".")
-        )
-
-
-def is_callable(obj):
-    return callable(obj)
+    return all(
+        part.isidentifier() and not keyword.iskeyword(part)
+        for part in string.split(".")
+    )
 
 
 def get_argspec_from_signature(f):
@@ -369,63 +340,30 @@ def get_encoding(obj):
         m = get_encoding_line_re.search(line)
         if m:
             return m.group(1)
-    return "ascii"
+    return "utf8"
 
 
-def get_encoding_comment(source):
-    """Returns encoding line without the newline, or None is not found"""
-    for line in source.splitlines()[:2]:
-        m = get_encoding_line_re.search(line)
-        if m:
-            return m.group(0)
-    return None
-
-
-def get_encoding_file(fname):
+def get_encoding_file(fname: str) -> str:
     """Try to obtain encoding information from a Python source file."""
-    with io.open(fname, "rt", encoding="ascii", errors="ignore") as f:
+    with open(fname, encoding="ascii", errors="ignore") as f:
         for unused in range(2):
             line = f.readline()
             match = get_encoding_line_re.search(line)
             if match:
                 return match.group(1)
-    return "ascii"
+    return "utf8"
 
 
-if not py3:
-
-    def getattr_safe(obj, name):
-        """side effect free getattr"""
-        if not is_new_style(obj):
-            return getattr(obj, name)
-
-        with AttrCleaner(obj):
-            to_look_through = (
-                obj.__mro__
-                if inspect.isclass(obj)
-                else (obj,) + type(obj).__mro__
-            )
-            for cls in to_look_through:
-                if hasattr(cls, "__dict__") and name in cls.__dict__:
-                    result = cls.__dict__[name]
-                    if isinstance(result, MemberDescriptorType):
-                        result = getattr(obj, name)
-                    return result
-            raise AttributeError(name)
+def getattr_safe(obj: Any, name: str):
+    """side effect free getattr (calls getattr_static)."""
+    result = inspect.getattr_static(obj, name)
+    # Slots are a MemberDescriptorType
+    if isinstance(result, MemberDescriptorType):
+        result = getattr(obj, name)
+    return result
 
 
-else:
-
-    def getattr_safe(obj, name):
-        """side effect free getattr (calls getattr_static)."""
-        result = inspect.getattr_static(obj, name)
-        # Slots are a MemberDescriptorType
-        if isinstance(result, MemberDescriptorType):
-            result = getattr(obj, name)
-        return result
-
-
-def hasattr_safe(obj, name):
+def hasattr_safe(obj: Any, name: str) -> bool:
     try:
         getattr_safe(obj, name)
         return True
@@ -433,15 +371,6 @@ def hasattr_safe(obj, name):
         return False
 
 
-if py3:
-
-    def get_source_unicode(obj):
-        """Returns a decoded source of object"""
-        return inspect.getsource(obj)
-
-
-else:
-
-    def get_source_unicode(obj):
-        """Returns a decoded source of object"""
-        return inspect.getsource(obj).decode(get_encoding(obj))
+def get_source_unicode(obj):
+    """Returns a decoded source of object"""
+    return inspect.getsource(obj)
