@@ -25,8 +25,7 @@ from bpython.test import FixLanguageTestCase as TestCase, TEST_CONFIG
 
 
 def setup_config():
-    config_struct = config.Struct()
-    config.loadini(config_struct, TEST_CONFIG)
+    config_struct = config.Config(TEST_CONFIG)
     config_struct.cli_suggestion_width = 1
     return config_struct
 
@@ -140,6 +139,14 @@ class TestCurtsiesPaintingSimple(CurtsiesPaintingTest):
                 "└──────────────────────────────┘",
                 "Welcome to bpython! Press <F1> f",
             ]
+            if sys.version_info[:2] < (3, 10)
+            else [
+                ">>> an",
+                "┌──────────────────────────────┐",
+                "│ and    anext( any(           │",
+                "└──────────────────────────────┘",
+                "Welcome to bpython! Press <F1> f",
+            ]
         )
         self.assert_paint_ignoring_formatting(screen, (0, 4))
 
